package com.qunzhixinxi.hnqz.admin.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ArrayUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qunzhixinxi.hnqz.admin.api.entity.SysDeptRelation;
import com.qunzhixinxi.hnqz.admin.entity.WmDaDrugEnt;
import com.qunzhixinxi.hnqz.admin.entity.WmDeptTaskTypeConfig;
import com.qunzhixinxi.hnqz.admin.entity.WmScorePackage;
import com.qunzhixinxi.hnqz.admin.entity.WmTask;
import com.qunzhixinxi.hnqz.admin.entity.WmTaskType;
import com.qunzhixinxi.hnqz.admin.entity.dto.WmDeptTaskTypeConfigDTO;
import com.qunzhixinxi.hnqz.admin.entity.dto.WmDeptTaskTypeConfigJsonDTO;
import com.qunzhixinxi.hnqz.admin.enums.DelEnum;
import com.qunzhixinxi.hnqz.admin.enums.EnableEnum;
import com.qunzhixinxi.hnqz.admin.enums.PackageTypeEnum;
import com.qunzhixinxi.hnqz.admin.enums.ScorePackageStatusEnum;
import com.qunzhixinxi.hnqz.admin.enums.TaskStatusEnum;
import com.qunzhixinxi.hnqz.admin.mapper.SysDeptRelationMapper;
import com.qunzhixinxi.hnqz.admin.mapper.WmDaDrugEntMapper;
import com.qunzhixinxi.hnqz.admin.mapper.WmDeptTaskTypeConfigMapper;
import com.qunzhixinxi.hnqz.admin.mapper.WmTaskMapper;
import com.qunzhixinxi.hnqz.admin.service.WmDeptTaskTypeConfigService;
import com.qunzhixinxi.hnqz.admin.service.WmScorePackageService;
import com.qunzhixinxi.hnqz.admin.service.WmTaskService;
import com.qunzhixinxi.hnqz.admin.service.WmTaskTypeService;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.security.util.SecurityUtils;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 部门任务类型配置服务
 *
 * @author lixuesong
 * @date 2021年12月03日 10:14
 */
@Slf4j
@AllArgsConstructor
@Service
public class WmDeptTaskTypeConfigServiceImpl extends ServiceImpl<WmDeptTaskTypeConfigMapper, WmDeptTaskTypeConfig> implements WmDeptTaskTypeConfigService {

	private final WmTaskTypeService wmTaskTypeService;

	private final WmScorePackageService wmScorePackageService;

	private final SysDeptRelationMapper sysDeptRelationMapper;

	private final WmDaDrugEntMapper wmDaDrugEntMapper;

	private final WmTaskMapper wmTaskMapper;

	/**
	 * 保存任务类型配置
	 *
	 * @param wmDeptTaskTypeConfigDTO
	 */
	@Override
	public void saveDeptTaskTypeConfig(WmDeptTaskTypeConfigDTO wmDeptTaskTypeConfigDTO) {
		// 判断是否已存在
		int count = this.count(Wrappers.<WmDeptTaskTypeConfig>lambdaQuery()
				.eq(WmDeptTaskTypeConfig::getDeptId, wmDeptTaskTypeConfigDTO.getDeptId()));
		if (count <= 0) {
			// 新增
			WmDeptTaskTypeConfig deptTaskTypeConfig = new WmDeptTaskTypeConfig();
			deptTaskTypeConfig.setDeptId(wmDeptTaskTypeConfigDTO.getDeptId());
			deptTaskTypeConfig.setTaskTypeJson(JSON.toJSONString(wmDeptTaskTypeConfigDTO.getConfigList()));
			deptTaskTypeConfig.setCreateTime(LocalDateTime.now());
			deptTaskTypeConfig.setCreateUser(SecurityUtils.getUser().getId());
			this.save(deptTaskTypeConfig);
		} else {
			// 更新
			WmDeptTaskTypeConfig deptTaskTypeConfig = new WmDeptTaskTypeConfig();
			deptTaskTypeConfig.setDeptId(wmDeptTaskTypeConfigDTO.getDeptId());
			deptTaskTypeConfig.setTaskTypeJson(JSON.toJSONString(wmDeptTaskTypeConfigDTO.getConfigList()));
			deptTaskTypeConfig.setUpdateTime(LocalDateTime.now());
			deptTaskTypeConfig.setUpdateUser(SecurityUtils.getUser().getId());
			this.update(Wrappers.<WmDeptTaskTypeConfig>lambdaUpdate()
					.eq(WmDeptTaskTypeConfig::getDeptId, deptTaskTypeConfig.getDeptId())
					.set(WmDeptTaskTypeConfig::getTaskTypeJson, JSON.toJSONString(wmDeptTaskTypeConfigDTO.getConfigList()))
					.set(WmDeptTaskTypeConfig::getUpdateTime, LocalDateTime.now())
					.set(WmDeptTaskTypeConfig::getUpdateUser, SecurityUtils.getUser().getId()));
		}
	}

	/**
	 * 查询任务类型配置
	 *
	 * @param deptId
	 * @return
	 */
	@Override
	public WmDeptTaskTypeConfigDTO getDeptTaskTypeConfig(Integer deptId) {
		String[] enableZbdbStatusArr = {"医院拜访", "商业公司拜访", "药店拜访", "电子名片分享", "产品相册分享", "学术信息交流",
				"受众打开产品相册", "受众阅读文章"};
		// 查询出当前dept对应的任务类型（药企或药企给CSO的任务类型）
		List<SysDeptRelation> relationList = sysDeptRelationMapper.selectList(Wrappers.<SysDeptRelation>lambdaQuery()
				.eq(SysDeptRelation::getDescendant, deptId));
		SysDeptRelation deptRelation = relationList.stream()
				.filter(sysDeptRelation -> !sysDeptRelation.getAncestor().equals(1))
				.findFirst()
				.get();
		WmDaDrugEnt wmDaDrugEnt = wmDaDrugEntMapper.selectOne(Wrappers.<WmDaDrugEnt>lambdaQuery()
				.eq(WmDaDrugEnt::getDeptId, deptRelation.getAncestor())
				.eq(WmDaDrugEnt::getEnableFlag, EnableEnum.ENABLE.val()));
		List<WmTaskType> taskTypeList = wmTaskTypeService.list(Wrappers.<WmTaskType>lambdaQuery()
				.eq(WmTaskType::getDeptId, deptId)
				.eq(WmTaskType::getDurgentId, wmDaDrugEnt.getId()));

		WmDeptTaskTypeConfig deptTaskTypeConfig = this.getOne(Wrappers.<WmDeptTaskTypeConfig>lambdaQuery()
				.eq(WmDeptTaskTypeConfig::getDeptId, deptId));

		WmDeptTaskTypeConfigDTO deptTaskTypeConfigDTO = new WmDeptTaskTypeConfigDTO();
		List<WmDeptTaskTypeConfigJsonDTO> deptTaskTypeConfigJsonDTOList = new ArrayList<>();
		deptTaskTypeConfigDTO.setDeptId(deptId);
		Map<String, WmDeptTaskTypeConfigJsonDTO> configMap = null;

		// 若配置任务类型不为空，则设置复选框的选择状态，否则设置复选框为默认状态
		if (deptTaskTypeConfig != null) {
			deptTaskTypeConfigDTO.setId(deptTaskTypeConfig.getId());
			List<WmDeptTaskTypeConfigJsonDTO> configJsonDTOList = JSON.parseArray(deptTaskTypeConfig.getTaskTypeJson(), WmDeptTaskTypeConfigJsonDTO.class);
			configMap = configJsonDTOList.stream()
					.collect(Collectors.toMap(WmDeptTaskTypeConfigJsonDTO::getTaskTypeName, wmDeptTaskTypeConfigJsonDTO -> wmDeptTaskTypeConfigJsonDTO));
		}
		// 以企业的任务类型数据为基础，进行任务类型配置数据的拼接
		if (CollectionUtil.isNotEmpty(taskTypeList)) {
			for (WmTaskType taskType : taskTypeList) {
				WmDeptTaskTypeConfigJsonDTO deptTaskTypeConfigJsonDTO = null;
				if (configMap != null) {
					// 若配置存在，则填充
					WmDeptTaskTypeConfigJsonDTO baseConfigJson = configMap.get(taskType.getTaskTypeName());
					deptTaskTypeConfigJsonDTO = BeanUtil.copyProperties(baseConfigJson, WmDeptTaskTypeConfigJsonDTO.class);
				} else {
					// 若配置不存在则，只设置默认值为未勾选状态
					deptTaskTypeConfigJsonDTO = new WmDeptTaskTypeConfigJsonDTO();
				}
				deptTaskTypeConfigJsonDTO.setTaskTypeId(taskType.getId());
				deptTaskTypeConfigJsonDTO.setTaskTypeName(taskType.getTaskTypeName());
				deptTaskTypeConfigJsonDTO.setScore(taskType.getScore());
				if (ArrayUtil.contains(enableZbdbStatusArr, taskType.getTaskTypeName())) {
					deptTaskTypeConfigJsonDTO.setZbdbStatusEnable(true);
				} else {
					deptTaskTypeConfigJsonDTO.setZbdbStatusEnable(false);
				}
				deptTaskTypeConfigJsonDTOList.add(deptTaskTypeConfigJsonDTO);
			}
			deptTaskTypeConfigDTO.setConfigList(deptTaskTypeConfigJsonDTOList);
		}

		return deptTaskTypeConfigDTO;
	}

	/**
	 * 根据企业的任务类型配置查询任务列表
	 *
	 * @param taskTypeName 任务类型名称
	 * @param taskStatus   任务状态
	 * @return
	 */
	@Override
	public IPage<WmTask> listTaskByDeptTaskType(Page<WmTask> page, String taskTypeName, String[] taskStatus) {
		Integer deptId = SecurityUtils.getUser().getDeptId();
		WmDeptTaskTypeConfig deptTaskTypeConfig = this.getOne(Wrappers.<WmDeptTaskTypeConfig>lambdaQuery()
				.eq(WmDeptTaskTypeConfig::getDeptId, deptId));
		if (deptTaskTypeConfig != null) {
			// 查询该企业当前任务类型下 的任务类型配置数据
			List<WmDeptTaskTypeConfigJsonDTO> configJsonDTOList =
					JSON.parseArray(deptTaskTypeConfig.getTaskTypeJson(), WmDeptTaskTypeConfigJsonDTO.class);
			WmDeptTaskTypeConfigJsonDTO taskTypeConfigJsonDTO = configJsonDTOList.stream()
					.filter(configJsonDTO -> configJsonDTO.getTaskTypeName().equals(taskTypeName))
					.findFirst()
					.get();
			// 查询任务名称对应的基础任务类型id
			WmTaskType baseTaskType = wmTaskTypeService.getOne(Wrappers.<WmTaskType>lambdaQuery()
					.eq(WmTaskType::getTaskTypeLevel, "1")
					.eq(WmTaskType::getTaskTypeName, taskTypeName)
					.eq(WmTaskType::getEnableFlag, EnableEnum.ENABLE.val()));
			// 根据是否勾选了关联下级，查询当前企业对应的积分包
			List<String> scorePackageIdList = new ArrayList<>();
			LambdaQueryWrapper<WmScorePackage> scorePackageWrapper = Wrappers.lambdaQuery();
			scorePackageWrapper.eq(WmScorePackage::getSendPackageDeptId, deptId);
			scorePackageWrapper.eq(WmScorePackage::getEnableFlag, EnableEnum.ENABLE.val());
			scorePackageWrapper.eq(WmScorePackage::getDelFlag, DelEnum.NOT_DEL.val());
			scorePackageWrapper.eq(WmScorePackage::getPackageType1, PackageTypeEnum.TYPE1_SCOTE_PACKAGE.val());
			scorePackageWrapper.eq(WmScorePackage::getPackageType2, PackageTypeEnum.TYPE2_TASK_PACKAGE.val());
			scorePackageWrapper.ge(WmScorePackage::getScorePackageStatus, ScorePackageStatusEnum.IN_PROGRESS.val());
			scorePackageWrapper.in(WmScorePackage::getTypeid, "3", "4");
			if (taskTypeConfigJsonDTO.getAssociateLower()) {
				// 配置了关联下级，则同时查询'对应机构下发的企业承接的任务，再下发给个人的积分包'
//				scorePackageWrapper.isNotNull(WmScorePackage::getAcceptSug);
//				scorePackageWrapper.apply("length(accept_sug) > 0");
				List<WmScorePackage> scorePackageList = wmScorePackageService.list(scorePackageWrapper);
				if (CollectionUtil.isNotEmpty(scorePackageList)) {
					// 遍历查找所有下级积分包
					Set<String> ids = scorePackageList.stream().map(WmScorePackage::getId).collect(Collectors.toSet());
					getAllLowerScorePackages(ids, scorePackageList);

					// 过滤只要发给个人的包
					scorePackageIdList = scorePackageList.stream()
							.filter(scorePackage -> StringUtils.isBlank(scorePackage.getAcceptSug()))
							.map(WmScorePackage::getId).collect(Collectors.toList());
				}
			} else {
				// 未勾选关联下级，则全部任务中只显示未关联积分包的任务，就是从企业直接下发给医药代表的任务 是未关联积分包吗？代理商可以有吧???
				scorePackageWrapper.and(wrapper -> wrapper.isNull(WmScorePackage::getAcceptSug)
						.or()
						.apply("length(accept_sug) = 0"));
				List<WmScorePackage> scorePackageList = wmScorePackageService.list(scorePackageWrapper);
				if (CollectionUtil.isNotEmpty(scorePackageList)) {
					scorePackageIdList = scorePackageList.stream()
							// 过滤只要发给个人的包
							.filter(scorePackage -> StringUtils.isBlank(scorePackage.getAcceptSug()))
							.map(WmScorePackage::getId).collect(Collectors.toList());
				}
			}
			// 用户类型
			List<String> taskUserTypeList = new ArrayList<>();
			if (taskTypeConfigJsonDTO.getZbdbStatusEnable() && taskTypeConfigJsonDTO.getZbdbStatusSelect()) {
				taskUserTypeList.add("5");
			}
			if (taskTypeConfigJsonDTO.getYydbStatusEnable() && taskTypeConfigJsonDTO.getZbdbStatusSelect()) {
				taskUserTypeList.add("6");
			}

			if (CollectionUtil.isNotEmpty(scorePackageIdList) && CollectionUtil.isNotEmpty(taskUserTypeList)) {
				// 查询任务列表
				WmTask queryTask = new WmTask();
				queryTask.setTaskTypeId(baseTaskType.getId());
				queryTask.setScorePackageId(String.join(",", scorePackageIdList));
				queryTask.setTaskUserType(String.join(",", taskUserTypeList));
				queryTask.setTaskStatus(String.join(",", taskStatus));
				IPage<WmTask> taskPage = wmTaskMapper.listWmTaskByConfig(page, queryTask);
				return taskPage;
			}
		}

		return page;
	}

	/**
	 * 查询所有下级的积分包
	 *
	 * @param ids
	 * @param res
	 */
	private void getAllLowerScorePackages(Set<String> ids, List<WmScorePackage> res) {
		List<WmScorePackage> subList = wmScorePackageService.list(Wrappers.<WmScorePackage>lambdaQuery()
				.in(WmScorePackage::getRelationScoreId, ids)
				.eq(WmScorePackage::getEnableFlag, EnableEnum.ENABLE.val()));

		if (CollectionUtil.isNotEmpty(subList)) {
			Set<String> idSet = subList.stream().map(WmScorePackage::getId).collect(Collectors.toSet());
			res.addAll(subList);
			getAllLowerScorePackages(idSet, res);
		}
	}
}
